import zoneinfo
from datetime import datetime

from bersona import Bersona

def test_basic_chart_generation():
    tz = zoneinfo.ZoneInfo("Asia/Shanghai")
    dt = datetime(1990, 5, 17, 14, 30, tzinfo=tz)
    b = Bersona()
    chart = b.generate_chart(dt, 31.2304, 121.4737)
    assert chart.planets, "Planets should not be empty"
    assert chart.input.birth_datetime == dt



"""Bersona package (src layout).

Public API: from bersona import Bersona
"""
from .astrology_kernel import Bersona  # re-export
from ._version import __version__, get_version

__all__ = ["Bersona", "__version__", "get_version"]



"""Central version definition.

Edit here (or use scripts/bump_version.py) when releasing.
"""
__all__ = ["__version__", "get_version"]
__version__ = "0.1.3"

def get_version() -> str:
    return __version__



# ...migrated content...
"""占星相关常量定义模块"""
from typing import Dict, List, Tuple

ZODIAC_SIGNS: List[Tuple[str, int]] = [
    ("Aries", 0), ("Taurus", 30), ("Gemini", 60), ("Cancer", 90),
    ("Leo", 120), ("Virgo", 150), ("Libra", 180), ("Scorpio", 210),
    ("Sagittarius", 240), ("Capricorn", 270), ("Aquarius", 300), ("Pisces", 330)
]

TRADITIONAL_RULERS: Dict[str, str] = {
    "Aries": "Mars", "Taurus": "Venus", "Gemini": "Mercury", "Cancer": "Moon",
    "Leo": "Sun", "Virgo": "Mercury", "Libra": "Venus", "Scorpio": "Mars",
    "Sagittarius": "Jupiter", "Capricorn": "Saturn", "Aquarius": "Saturn", "Pisces": "Jupiter"
}

MODERN_RULERS: Dict[str, str] = {
    **TRADITIONAL_RULERS,
    "Scorpio": "Pluto",
    "Aquarius": "Uranus",
    "Pisces": "Neptune"
}

MAJOR_ASPECTS_DEFAULT_ORBS: Dict[str, float] = {
    "Conjunction": 8.0,
    "Opposition": 8.0,
    "Trine": 7.0,
    "Square": 6.0,
    "Sextile": 4.0,
}

ASPECT_DEGREES: Dict[str, float] = {
    "Conjunction": 0.0,
    "Sextile": 60.0,
    "Square": 90.0,
    "Trine": 120.0,
    "Opposition": 180.0,
}

PLANET_NAMES: List[str] = [
    "Sun", "Moon", "Mercury", "Venus", "Mars", "Jupiter", "Saturn", "Uranus", "Neptune", "Pluto"
]



# ...migrated content...
"""Pydantic 数据模型定义"""
from __future__ import annotations
from datetime import datetime
from typing import List, Dict, Optional, Tuple, Any
from pydantic import BaseModel, Field, field_validator

class ChartInput(BaseModel):
    birth_datetime: datetime = Field(description="出生时间，需 timezone-aware")
    latitude: float = Field(ge=-90, le=90, description="纬度")
    longitude: float = Field(ge=-180, le=180, description="经度 (东正西负)")
    house_system: str = Field(default='equal', description="宫位系统 'equal' 或 'placidus'")
    rulers_scheme: str = Field(default='traditional', description="主宰体系 'traditional' 或 'modern'")
    aspect_orbs: Optional[Dict[str, float]] = Field(default=None, description="相位容许度覆盖表")
    date_only: bool = Field(default=False, description="输入是否为仅日期（自动补 12:00），若 True 则不计算 Asc 与 Houses")

    @field_validator('birth_datetime')
    def _tz_required(cls, v: datetime):
        if v.tzinfo is None:
            raise ValueError('datetime 必须带时区信息')
        return v

    @field_validator('house_system')
    def _house_system_ok(cls, v: str):
        if v not in ('equal', 'placidus'):
            raise ValueError("house_system 仅支持 'equal' 或 'placidus'")
        return v

    @field_validator('rulers_scheme')
    def _rulers_scheme_ok(cls, v: str):
        if v not in ('traditional', 'modern'):
            raise ValueError("rulers_scheme 仅支持 'traditional' 或 'modern'")
        return v

class Ascendant(BaseModel):
    longitude: float
    sign: str

class HouseCusp(BaseModel):
    house: int = Field(ge=1, le=12)
    cusp_longitude: float
    cusp_sign: str

class PlanetPosition(BaseModel):
    name: str
    ecliptic_longitude: float
    ecliptic_latitude: float
    sign: str
    retrograde: bool

class Aspect(BaseModel):
    planet1: str
    planet2: str
    aspect: str
    separation: float = Field(description="两行星最小分离角 (0-180)")
    difference: float = Field(description="与标准相位角的差值")
    orb_allowed: float

class MutualReception(BaseModel):
    planet1: str
    planet2: str
    scheme: str
    signs: Tuple[str, str]

class ChartSettings(BaseModel):
    house_system: str
    rulers_scheme: str
    aspect_orbs: Dict[str, float]
    libraries: Dict[str, bool]

class ChartResult(BaseModel):
    input: ChartInput
    settings: ChartSettings
    ascendant: Optional[Ascendant] = None
    houses: List[HouseCusp] = []
    planets: Dict[str, PlanetPosition]
    aspects: List[Aspect]
    mutual_receptions: List[MutualReception]

    def summary(self) -> Dict[str, int]:
        return {
            'planets': len(self.planets),
            'aspects': len(self.aspects),
            'mutual_receptions': len(self.mutual_receptions)
        }


class AstrologyDesc(BaseModel):
    """LLM 生成的占星解释描述。"""
    text: str = Field(description="自然语言描述")
    model_used: Optional[str] = Field(default=None, description="使用的 LLM 模型名称")
    created_at: datetime = Field(default_factory=datetime.utcnow, description="UTC 创建时间")
    language: str = Field(default='zh', description="描述语言标记")
    chart_snapshot: Dict[str, Any] = Field(default_factory=dict, description="星盘摘要或关键信息快照")

    def short(self) -> str:
        return self.text[:120] + ('...' if len(self.text) > 120 else '')



from datetime import datetime, timedelta, timezone
import re
from typing import Dict, Any, List, Optional
import os
import logging
# 初始化日志
logger = logging.getLogger("bersona")
if not logger.handlers:
    _handler = logging.StreamHandler()
    _formatter = logging.Formatter("[%(asctime)s] %(levelname)s %(name)s: %(message)s")
    _handler.setFormatter(_formatter)
    logger.addHandler(_handler)
log_level = os.getenv("BERSONA_LOG_LEVEL", "INFO").upper()
try:
    logger.setLevel(getattr(logging, log_level, logging.INFO))
except Exception:
    logger.setLevel(logging.INFO)

from .constants import (
    ZODIAC_SIGNS,
    TRADITIONAL_RULERS,
    MODERN_RULERS,
    MAJOR_ASPECTS_DEFAULT_ORBS,
    ASPECT_DEGREES,
    PLANET_NAMES,
)
from .utils import angle_to_sign, angular_distance, parse_birth_datetime, chart_to_text
from .prompts import BASE_PROMPTS
from .models import (
    ChartInput,
    ChartSettings,
    ChartResult,
    Ascendant,
    HouseCusp,
    PlanetPosition,
    Aspect,
    MutualReception,
    AstrologyDesc,
)

try:
    from skyfield.api import load, wgs84
    _EPHEMERIS_NAME = os.getenv('BERSONA_EPHEMERIS', 'de421.bsp')
    cache_dir = os.getenv('SKYFIELD_CACHE_DIR')
    if cache_dir:
        os.makedirs(cache_dir, exist_ok=True)
        from skyfield.api import Loader
        loader = Loader(cache_dir)
        _TS = loader.timescale()
        ephemeris_path = os.path.join(cache_dir, _EPHEMERIS_NAME)
        if not os.path.exists(ephemeris_path):
            approx_size_mb = '20' if 'de421' in _EPHEMERIS_NAME else ('120' if 'de440' in _EPHEMERIS_NAME else '若干')
            logger.info(f"[Bersona] 尚未发现星历文件 '{_EPHEMERIS_NAME}'，首次使用将自动下载约 {approx_size_mb}MB，需保持网络畅通...")
        _EPHEMERIS = loader(_EPHEMERIS_NAME)
    else:
        _TS = load.timescale()
        home_cache = os.path.join(os.path.expanduser('~'), '.skyfield', _EPHEMERIS_NAME)
        if not os.path.exists(home_cache):
            approx_size_mb = '20' if 'de421' in _EPHEMERIS_NAME else ('120' if 'de440' in _EPHEMERIS_NAME else '若干')
            logger.info(f"[Bersona] 首次使用星历 '{_EPHEMERIS_NAME}'，将自动下载约 {approx_size_mb}MB，需保持网络畅通...")
        _EPHEMERIS = load(_EPHEMERIS_NAME)
    _PLANETS = {
        "Sun": _EPHEMERIS["sun"],
        "Moon": _EPHEMERIS["moon"],
        "Mercury": _EPHEMERIS["mercury"],
        "Venus": _EPHEMERIS["venus"],
        "Mars": _EPHEMERIS["mars"],
        "Jupiter": _EPHEMERIS["jupiter barycenter"],
        "Saturn": _EPHEMERIS["saturn barycenter"],
        "Uranus": _EPHEMERIS["uranus barycenter"],
        "Neptune": _EPHEMERIS["neptune barycenter"],
        "Pluto": _EPHEMERIS["pluto barycenter"],
    }
    _SKYFIELD_AVAILABLE = True
    logger.debug("Skyfield 已加载: %s", _EPHEMERIS_NAME)
except Exception as e:
    logger.warning("Skyfield 加载失败: %s", e)
    _SKYFIELD_AVAILABLE = False

try:
    import swisseph as swe
    _SWISSEPH_AVAILABLE = True
    logger.debug("SwissEph 已加载")
except Exception as e:
    logger.warning("SwissEph 加载失败: %s", e)
    _SWISSEPH_AVAILABLE = False

class Bersona:
    # 新增: 默认语言类属性（可按需扩展）
    DEFAULT_LANGUAGE = 'zh'

    def __init__(self,
                 llm_client: Any = None,
                 llm_model: Optional[str] = None,
                 system_prompt: Optional[str] = None) -> None:
        """
        可选参数:
        llm_client: 传入已构造好的 OpenAI 客户端(或兼容接口对象)，则不再内部自动创建
        llm_model: 指定使用的模型；若为空且使用环境变量模式，则读取 OPENAI_MODEL
        system_prompt: 覆盖默认的 system prompt
        """
        self.available_skyfield = _SKYFIELD_AVAILABLE
        self.available_swisseph = _SWISSEPH_AVAILABLE
        self._llm_client = None
        self._llm_model = None

        default_lang = os.getenv("BERSONA_DEFAULT_LANG", self.DEFAULT_LANGUAGE)
        # 若显式传入 system_prompt 则使用；否则按语言默认
        self.system_prompt: Optional[str] = system_prompt or BASE_PROMPTS.get(default_lang, BASE_PROMPTS[self.DEFAULT_LANGUAGE])

        logger.debug("初始化 Bersona: skyfield=%s swisseph=%s", self.available_skyfield, self.available_swisseph)

        if llm_client is not None:
            # 外部注入模式
            self._llm_client = llm_client
            self._llm_model = llm_model or os.getenv('OPENAI_MODEL')
            self.llm_available = True
            logger.info("使用外部注入的 LLM 客户端，模型=%s", self._llm_model)
        else:
            # 保留原环境变量自动创建逻辑
            api_key = None
            base_url = None
            try:
                api_key = os.getenv('OPENAI_API_KEY') or os.getenv('OPENAI_KEY')
                base_url = os.getenv('OPENAI_BASE_URL')
            except Exception as e:
                logger.debug("读取 OpenAI 环境变量失败: %s", e)
                api_key = None
            if api_key:
                try:
                    from openai import OpenAI
                    kwargs = {'api_key': api_key}
                    if base_url:
                        kwargs['base_url'] = base_url
                    self._llm_client = OpenAI(**kwargs)
                    self._llm_model = os.getenv('OPENAI_MODEL')
                    logger.info("LLM 客户端初始化成功，模型=%s", self._llm_model)
                except Exception as e:
                    logger.warning("LLM 初始化失败: %s", e)
                    self._llm_client = None
            self.llm_available = self._llm_client is not None

    # 新增: 设置 / 覆盖实例级 system prompt 的便捷方法
    def set_system_prompt(self, prompt: str) -> None:
        """
        设置实例级的 system prompt。传入空字符串将视为清除，自行回退到 BASE_PROMPTS 逻辑。
        """
        self.system_prompt = prompt or None

    def set_llm_client(self, client: Any, model: Optional[str] = None) -> None:
        """
        动态注入 / 替换 LLM 客户端。
        client: 需提供 chat.completions.create 接口的对象
        model: 若指定则更新模型；否则保留原值或再从环境变量读取
        """
        self._llm_client = client
        if model:
            self._llm_model = model
        elif not self._llm_model:
            self._llm_model = os.getenv('OPENAI_MODEL')
        self.llm_available = self._llm_client is not None
        logger.info("已更新 LLM 客户端，当前模型=%s", self._llm_model)

    def llm_chat(self, messages: List[Dict[str, str]], model: Optional[str] = None) -> Optional[str]:
        if not self.llm_available:
            logger.warning("LLM 不可用，跳过 llm_chat 调用")
            return None
        use_model = model or self._llm_model
        if not use_model:
            logger.error("未指定模型且环境变量 OPENAI_MODEL 未设置")
            raise ValueError('未指定模型名称，且环境变量 OPENAI_MODEL 未设置')
        logger.debug("调用 LLM: model=%s", use_model)
        try:
            resp = self._llm_client.chat.completions.create(
                model=use_model,
                messages=messages
            )
            if resp and resp.choices:
                content = resp.choices[0].message.content
                logger.debug("LLM 返回长度=%d", len(content or ""))
                return content
            logger.warning("LLM 无 choices 返回")
        except Exception as e:
            logger.error("LLM 调用异常: %s", e)
            return None
        return None

    def astrology_describe(self,
                           chart: ChartResult,
                           model: Optional[str] = None,
                           language: str = 'zh',
                           system_prompt: Optional[str] = None) -> AstrologyDesc:
        """调用 LLM 生成占星描述，并兼容多种包裹格式提取描述正文。
        支持多种输出标记。
        优先级：显式参数 system_prompt > 实例属性 self.system_prompt > 语言默认 BASE_PROMPTS。
        """
        if not self.llm_available:
            raise RuntimeError('LLM 不可用：请设置 OPENAI_API_KEY 并确保网络可访问。')

        # 修改: 使用实例属性 system_prompt
        if system_prompt:
            base_prompt = system_prompt
        elif self.system_prompt:
            base_prompt = self.system_prompt
        else:
            base_prompt = BASE_PROMPTS.get(language.split('-')[0], BASE_PROMPTS['en'])

        chart_text = chart_to_text(chart)
        messages = [
            {'role': 'system', 'content': base_prompt},
            {'role': 'user', 'content': chart_text},
        ]
        response = self.llm_chat(messages, model=model)
        if not response:
            raise RuntimeError('LLM 调用失败或返回空响应。')

        raw = response.strip()
        desc_extracted: str = raw

        # 尝试模式 1 / 2：宽松匹配带反引号的开始与结束
        patterns = [
            # 代码块语言标记，开始与结束均为独立代码块语言
            re.compile(r"```ASTROLOGY_DESC_START\s*\n(.*?)\n```ASTROLOGY_DESC_END", re.DOTALL),
            # 紧贴三反引号的单行写法
            re.compile(r"```ASTROLOGY_DESC_START```(.*?)```ASTROLOGY_DESC_END```", re.DOTALL),
            # 开始为语言标记，结束为普通反引号 (部分模型可能只给结束的 ``` )
            re.compile(r"```ASTROLOGY_DESC_START\s*\n(.*?)\n```", re.DOTALL),
            # 无反引号纯文本标记
            re.compile(r"ASTROLOGY_DESC_START(.*?)ASTROLOGY_DESC_END", re.DOTALL),
        ]
        for idx, pat in enumerate(patterns, start=1):
            m = pat.search(raw)
            if m:
                desc_extracted = m.group(1).strip()
                logger.debug("占星描述匹配成功: pattern=%d 长度=%d", idx, len(desc_extracted))
                break
        else:
            logger.warning("未匹配到任何 ASTROLOGY_DESC 标记模式，使用完整响应正文")

        # 去除可能残留的首尾反引号块
        if desc_extracted.startswith("```") and desc_extracted.endswith("```"):
            desc_extracted = desc_extracted[3:-3].strip()

        # 构建一个轻量快照（避免巨大 JSON）
        try:
            chart_snapshot = {
                'ascendant': chart.ascendant.sign if chart.ascendant else None,
                'planets': {k: v.sign for k, v in chart.planets.items()},
                'aspects_count': len(chart.aspects),
            }
        except Exception:
            chart_snapshot = {}

        return AstrologyDesc(
            text=desc_extracted,
            model_used=model or self._llm_model,
            language=language,
            chart_snapshot=chart_snapshot,
        )

    def generate_chart(self,
                       birth_dt_input: Any,
                       latitude: float = 39.9042,
                       longitude: float = 116.4074,
                       house_system: str = 'placidus',
                       aspect_orbs: Optional[Dict[str, float]] = None,
                       rulers_scheme: str = 'traditional') -> ChartResult:
        logger.info("开始生成星盘: raw_input=%s lat=%.4f lon=%.4f house_system=%s rulers_scheme=%s",
                    birth_dt_input, latitude, longitude, house_system, rulers_scheme)
        raw_input = birth_dt_input
        date_only_flag = False
        if isinstance(raw_input, str):
            date_only_patterns = [
                r"^\d{4}-\d{2}-\d{2}$",
                r"^\d{4}/\d{2}/\d{2}$",
                r"^\d{4}年\d{1,2}月\d{1,2}日$",
            ]
            for pat in date_only_patterns:
                if re.match(pat, raw_input.strip()):
                    date_only_flag = True
                    if '年' in raw_input:
                        m = re.match(r"^(\d{4})年(\d{1,2})月(\d{1,2})日$", raw_input.strip())
                        y, mo, d = m.groups()
                        raw_input = f"{y}-{int(mo):02d}-{int(d):02d} 12:00:00"
                    elif '/' in raw_input:
                        parts = raw_input.strip().split('/')
                        raw_input = f"{parts[0]}-{parts[1]}-{parts[2]} 12:00:00"
                    else:
                        raw_input = raw_input.strip() + ' 12:00:00'
                    break
        birth_dt = parse_birth_datetime(raw_input)
        logger.debug("解析时间结果: %s (tz=%s)", birth_dt, birth_dt.tzinfo)
        if birth_dt.tzinfo is None:
            raise ValueError("birth_dt 必须为带时区的 datetime")
        if house_system not in ('equal', 'placidus'):
            raise ValueError("house_system 仅支持 'equal'|'placidus'")
        if rulers_scheme not in ('traditional', 'modern'):
            raise ValueError("rulers_scheme 仅支持 'traditional'|'modern'")

        aspect_orbs = aspect_orbs or MAJOR_ASPECTS_DEFAULT_ORBS
        rulers_map = TRADITIONAL_RULERS if rulers_scheme == 'traditional' else MODERN_RULERS

        input_model = ChartInput(
            birth_datetime=birth_dt,
            latitude=latitude,
            longitude=longitude,
            house_system=house_system,
            rulers_scheme=rulers_scheme,
            aspect_orbs=aspect_orbs,
            date_only=date_only_flag,
        )
        settings_model = ChartSettings(
            house_system=house_system,
            rulers_scheme=rulers_scheme,
            aspect_orbs=aspect_orbs,
            libraries={'skyfield': self.available_skyfield, 'pyswisseph': self.available_swisseph},
        )

        ascendant_model: Optional[Ascendant] = None
        houses_list: List[HouseCusp] = []
        planets_dict: Dict[str, PlanetPosition] = {}
        aspects_list: List[Aspect] = []
        mutual_receptions_list: List[MutualReception] = []

        if self.available_skyfield:
            t = _TS.from_datetime(birth_dt)
        else:
            t = None

        ascendant_model = None
        if not date_only_flag:
            logger.debug("计算宫位: system=%s swisseph=%s", house_system, self.available_swisseph)
            asc_long: float
            if house_system == 'placidus' and self.available_swisseph:
                dt_utc = birth_dt.astimezone(timezone.utc)
                ut_hour = dt_utc.hour + dt_utc.minute / 60 + dt_utc.second / 3600
                jd_ut = swe.julday(dt_utc.year, dt_utc.month, dt_utc.day, ut_hour, swe.GREG_CAL)
                try:
                    cusps, ascmc = swe.houses(jd_ut, latitude, longitude, b'P')
                    asc_long = float(ascmc[0]) % 360
                    for i in range(1, 13):
                        cusp = float(cusps[i]) % 360
                        houses_list.append(HouseCusp(house=i, cusp_longitude=cusp, cusp_sign=angle_to_sign(cusp)))
                except Exception:
                    if not self.available_skyfield or t is None:
                        asc_long = None
                    else:
                        gast_hours = t.gast
                        asc_long = (gast_hours * 15 + longitude) % 360
                        for i in range(1, 13):
                            start = (asc_long + (i - 1) * 30) % 360
                            houses_list.append(HouseCusp(house=i, cusp_longitude=start, cusp_sign=angle_to_sign(start)))
            else:
                if not self.available_skyfield or t is None:
                    raise RuntimeError('缺少精确时间与 Skyfield，不计算等宫制宫位')
                gast_hours = t.gast
                lst_deg = (gast_hours * 15 + longitude) % 360
                asc_long = lst_deg
                for i in range(1, 13):
                    start = (asc_long + (i - 1) * 30) % 360
                    houses_list.append(HouseCusp(house=i, cusp_longitude=start, cusp_sign=angle_to_sign(start)))
            if 'asc_long' in locals() and asc_long is not None:
                ascendant_model = Ascendant(longitude=asc_long, sign=angle_to_sign(asc_long))
                logger.info("上升星座: %s %.2f°", ascendant_model.sign, ascendant_model.longitude)

        planet_longitudes: Dict[str, float] = {}
        if not self.available_skyfield:
            logger.error("Skyfield 不可用，无法计算行星位置")
            raise RuntimeError('Skyfield 不可用，无法计算行星位置。')
        if t is not None:
            earth = _EPHEMERIS['earth']
            prev_dt = birth_dt - timedelta(days=1)
            t_prev = _TS.from_datetime(prev_dt)
            for name, planet in _PLANETS.items():
                try:
                    astrometric = earth.at(t).observe(planet).apparent()
                    lat_ecl, lon_ecl, _ = astrometric.ecliptic_latlon()
                    lon_deg = float(lon_ecl.degrees) % 360
                    lat_deg = float(lat_ecl.degrees)
                    astrometric_prev = earth.at(t_prev).observe(planet).apparent()
                    lat_prev, lon_prev, _ = astrometric_prev.ecliptic_latlon()
                    lon_prev_deg = float(lon_prev.degrees) % 360
                    diff_raw = lon_deg - lon_prev_deg
                    if diff_raw < -180:
                        diff_raw += 360
                    elif diff_raw > 180:
                        diff_raw -= 360
                    retrograde = diff_raw < 0
                    planet_longitudes[name] = lon_deg
                    planets_dict[name] = PlanetPosition(
                        name=name,
                        ecliptic_longitude=lon_deg,
                        ecliptic_latitude=lat_deg,
                        sign=angle_to_sign(lon_deg),
                        retrograde=retrograde,
                    )
                    logger.debug("行星位置: %s lon=%.4f lat=%.4f sign=%s R=%s",
                                 name, lon_deg, lat_deg, angle_to_sign(lon_deg), retrograde)
                except Exception as e:
                    logger.warning("行星计算失败 %s: %s", name, e)

        planet_names = list(planets_dict.keys())
        for i in range(len(planet_names)):
            for j in range(i + 1, len(planet_names)):
                p1 = planet_names[i]
                p2 = planet_names[j]
                lon1 = planet_longitudes[p1]
                lon2 = planet_longitudes[p2]
                separation = angular_distance(lon1, lon2)
                for aspect_name, aspect_deg in ASPECT_DEGREES.items():
                    orb_allowed = aspect_orbs.get(aspect_name, MAJOR_ASPECTS_DEFAULT_ORBS.get(aspect_name, 0))
                    diff = abs(separation - aspect_deg)
                    if diff <= orb_allowed:
                        aspects_list.append(Aspect(
                            planet1=p1,
                            planet2=p2,
                            aspect=aspect_name,
                            separation=separation,
                            difference=diff,
                            orb_allowed=orb_allowed,
                        ))
                        logger.debug("发现相位: %s-%s %s 分离=%.2f 差值=%.2f 允许容许=%.2f",
                                     p1, p2, aspect_name, separation, diff, orb_allowed)
        logger.info("相位数量: %d", len(aspects_list))

        for i in range(len(planet_names)):
            for j in range(i + 1, len(planet_names)):
                p1 = planet_names[i]
                p2 = planet_names[j]
                sign1 = planets_dict[p1].sign
                sign2 = planets_dict[p2].sign
                ruler1 = rulers_map.get(sign1)
                ruler2 = rulers_map.get(sign2)
                if ruler1 == p2 and ruler2 == p1 and p1 != p2:
                    mutual_receptions_list.append(MutualReception(
                        planet1=p1,
                        planet2=p2,
                        scheme=rulers_scheme,
                        signs=(sign1, sign2)
                    ))
                    logger.debug("发现互容: %s <-> %s (%s/%s)", p1, p2, sign1, sign2)
        logger.info("互容数量: %d", len(mutual_receptions_list))
        chart_result = ChartResult(
            input=input_model,
            settings=settings_model,
            ascendant=ascendant_model,
            houses=houses_list,
            planets=planets_dict,
            aspects=aspects_list,
            mutual_receptions=mutual_receptions_list,
        )
        logger.info("星盘生成完成")
        return chart_result

if __name__ == "__main__":
    import zoneinfo
    tz = zoneinfo.ZoneInfo("Asia/Shanghai")
    dt = datetime(1990, 5, 17, 14, 30, tzinfo=tz)
    b = Bersona()
    chart = b.generate_chart(dt, 31.2304, 121.4737)
    logger.info("示例星盘 JSON 输出")
    print(chart.model_dump_json(indent=2, ensure_ascii=False))



# ...migrated content...
"""占星解释 LLM 提示模板。

提供不同语言的基础系统提示。后续可扩展分层/风格/长度参数。
"""
from typing import Dict

__all__ = ["BASE_PROMPTS"]

BASE_PROMPTS: Dict[str, str] = {
    "zh": (
        "请基于后续提供的完整本命星盘原始数据，输出结构化且自然流畅的中文解释。"
        "需要描述这个人的性格、特质、喜好、职业方向等个人方面的特征，以及可能会感兴趣的媒体内容类型和风格。"
        "需要隐藏关于占星术语的表达，让这个描述能让普通人看懂。正面与反面的信息都需要包含。"
        "输出部分用```ASTROLOGY_DESC_START```和```ASTROLOGY_DESC_END```包裹。"
    ),
    "en": (
        "Using the full natal chart raw data provided, produce a structured yet fluid English interpretation."
        "Describe the person's personality, traits, preferences, career directions, and types and styles of media content they might be interested in."
        "Avoid astrological jargon, making the description accessible to a general audience. Include both positive and negative aspects."
        "Wrap the output section with ```ASTROLOGY_DESC_START``` and ```ASTROLOGY_DESC_END```."
    ),
}



# ...migrated content...
"""占星计算与辅助工具函数

新增:
    - parse_birth_datetime: 支持多种输入格式统一为 timezone-aware datetime
    - get_city_coordinates: 根据中国城市名称(中英文/拼音)返回经纬度
"""
from typing import Optional, Union, Dict
from datetime import datetime, timezone, timedelta
import re
import json
import os
from .constants import ZODIAC_SIGNS

__all__ = [
    "angle_to_sign", "normalize_angle", "angular_distance",
    "parse_birth_datetime", "get_city_coordinates",
    "parse_admin_location", "geocode_china_location"
]
__all__.append("chart_to_text")

def normalize_angle(angle: float) -> float:
    """归一化角度到 0-360"""
    return angle % 360


def angle_to_sign(angle: float) -> str:
    a = normalize_angle(angle)
    for name, start in ZODIAC_SIGNS:
        if start <= a < start + 30:
            return name
    return "Aries"  # 理论上不会到这里


def angular_distance(a1: float, a2: float) -> float:
    """返回两个角度的最小分离 (0-180)"""
    diff = abs(normalize_angle(a2) - normalize_angle(a1)) % 360
    return diff if diff <= 180 else 360 - diff

# ---------------- ChartResult 转文本 ----------------

def chart_to_text(chart) -> str:
    """将 ChartResult 全量结构序列化为可嵌入 LLM 提示的纯文本。

    输出包含:
      - Ascendant
      - Houses (编号 + 黄经 + 星座)
      - Planets (名称 + 黄道经纬度 + 星座 + 逆行)
      - Aspects (行星1-行星2 相位 分离差值 容许度)
      - Mutual Receptions
    """
    lines = []
    lines.append(f"Ascendant: sign={chart.ascendant.sign} longitude={chart.ascendant.longitude:.2f}")
    lines.append("Houses:")
    for h in chart.houses:
        lines.append(f"  House {h.house}: {h.cusp_longitude:.2f}° {h.cusp_sign}")
    lines.append("Planets:")
    for name, p in chart.planets.items():
        lines.append(f"  {name}: lon={p.ecliptic_longitude:.2f}° lat={p.ecliptic_latitude:.2f}° sign={p.sign} retrograde={p.retrograde}")
    lines.append("Aspects:")
    for a in chart.aspects:
        lines.append(f"  {a.planet1}-{a.planet2} {a.aspect} sep={a.separation:.2f} diff={a.difference:.2f} orb={a.orb_allowed:.2f}")
    lines.append("MutualReceptions:")
    for m in chart.mutual_receptions:
        lines.append(f"  {m.planet1}<->{m.planet2} signs={m.signs[0]}/{m.signs[1]} scheme={m.scheme}")
    return "\n".join(lines)

# ---------------- 时间解析工具 ----------------

ISO_PATTERNS = [
    # 2025-10-12T14:30:00+08:00 / 2025-10-12 14:30:00 / 2025/10/12 14:30
    r"^(\d{4}-\d{2}-\d{2})[T\s](\d{2}:\d{2}:\d{2})([+-]\d{2}:?\d{2})?$",
    r"^(\d{4}-\d{2}-\d{2})[T\s](\d{2}:\d{2})([+-]\d{2}:?\d{2})?$",
    r"^(\d{4}/\d{2}/\d{2})[\s](\d{2}:\d{2})([+-]\d{2}:?\d{2})?$",
]

CHINESE_DATE_PATTERN = r"^(\d{4})年(\d{1,2})月(\d{1,2})日\s?(\d{1,2})时?(\d{1,2})?分?(\d{1,2})?秒?(?:\s*([+-]\d{2}:?\d{2}))?$"

def _parse_offset(tz_str: Optional[str]) -> timezone:
    if not tz_str:
        # 默认使用 +08:00 (中国标准时间) 可按需改成 UTC
        return timezone.utc if False else timezone(timedelta(hours=8))
    # 形如 +0800 / +08:00 / -0530
    m = re.match(r"([+-])(\d{2}):?(\d{2})", tz_str)
    if not m:
        return timezone.utc
    sign = 1 if m.group(1) == '+' else -1
    hh = int(m.group(2))
    mm = int(m.group(3))
    return timezone(timedelta(hours=sign*hh, minutes=sign*mm))

def parse_birth_datetime(value: Union[str, int, float, datetime]) -> datetime:
    """将多种输入格式转换为 timezone-aware datetime。

    支持:
      - 已有 timezone-aware datetime: 直接返回
      - naive datetime: 自动补上默认时区 (+08:00)
      - ISO 字符串: 2025-10-12T14:30:00+08:00 / 2025-10-12 14:30:00
      - 简化字符串: 2025/10/12 14:30
      - 中文格式: 2025年10月12日14时30分 (可含秒与偏移)
      - Unix 时间戳 (int/float): 视为秒，默认转换为 UTC 再转 +08:00
    """
    if isinstance(value, datetime):
        return value if value.tzinfo else value.replace(tzinfo=timezone(timedelta(hours=8)))
    if isinstance(value, (int, float)):
        dt_utc = datetime.fromtimestamp(value, tz=timezone.utc)
        return dt_utc.astimezone(timezone(timedelta(hours=8)))
    if not isinstance(value, str):
        raise TypeError("不支持的日期输入类型")

    s = value.strip()
    # ISO / 常见格式
    for pat in ISO_PATTERNS:
        m = re.match(pat, s)
        if m:
            date_part = m.group(1).replace('/', '-')
            time_part = m.group(2)
            tz_part = m.group(3)
            if ':' not in time_part:
                raise ValueError('时间格式不符合预期')
            # 若无秒补 00
            if len(time_part.split(':')) == 2:
                time_part += ':00'
            tzinfo = _parse_offset(tz_part)
            dt = datetime.fromisoformat(f"{date_part} {time_part}")
            return dt.replace(tzinfo=tzinfo)

    # 中文格式
    m = re.match(CHINESE_DATE_PATTERN, s)
    if m:
        year, month, day = int(m.group(1)), int(m.group(2)), int(m.group(3))
        hour = int(m.group(4)) if m.group(4) else 0
        minute = int(m.group(5)) if m.group(5) else 0
        second = int(m.group(6)) if m.group(6) else 0
        tz_part = m.group(7)
        tzinfo = _parse_offset(tz_part)
        return datetime(year, month, day, hour, minute, second, tzinfo=tzinfo)

    # 直接尝试 python fromisoformat (可能包含时区)
    try:
        dt = datetime.fromisoformat(s)
        if dt.tzinfo is None:
            dt = dt.replace(tzinfo=timezone(timedelta(hours=8)))
        return dt
    except Exception:
        pass

    raise ValueError(f"无法解析日期字符串: {value}")

# ---------------- 城市坐标工具 ----------------


_GEOCODER = None
_GEOCODE_CACHE: Dict[str, tuple] = {}
# ---------------- 行政区规范解析与地理编码 ----------------

PROVINCE_KEYWORDS = [
    '北京市','天津市','上海市','重庆市',
    '河北省','山西省','辽宁省','吉林省','黑龙江省','江苏省','浙江省','安徽省','福建省','江西省','山东省','河南省','湖北省','湖南省','广东省','海南省',
    '四川省','贵州省','云南省','陕西省','甘肃省','青海省','台湾省',
    '内蒙古自治区','广西壮族自治区','西藏自治区','宁夏回族自治区','新疆维吾尔自治区','香港特别行政区','澳门特别行政区'
]

def parse_admin_location(s: str) -> Dict[str,str]:
    """解析输入格式: 省/自治区/直辖市 + 市/州。

    期望格式示例:
      - 浙江省杭州市
      - 北京市 (直辖市可只写一个)
      - 内蒙古自治区呼和浩特市
    返回: {'province':..., 'city':...}
    若无法解析抛出 ValueError。
    """
    if not s:
        raise ValueError('空的地点输入')
    s = s.strip()
    # 尝试匹配已知省/自治区关键字前缀
    for prov in sorted(PROVINCE_KEYWORDS, key=len, reverse=True):
        if s.startswith(prov):
            rest = s[len(prov):]
            if prov.endswith('市') and not rest:  # 直辖市无需 city 再次指定
                return {'province': prov, 'city': prov[:-1]}  # 去掉 “市” 作为城市名
            # 常见市/州/地区后缀
            m = re.match(r'(.+?)(市|州|地区|盟)$', rest)
            if m:
                city = m.group(1)
                return {'province': prov, 'city': city}
            raise ValueError(f'未识别城市部分: {rest}')
    # 若是仅一个直辖市名称（如 北京市）
    m2 = re.match(r'^(北京|天津|上海|重庆)市$', s)
    if m2:
        city_base = m2.group(1)
        return {'province': f'{city_base}市', 'city': city_base}
    raise ValueError(f'无法解析行政区地点: {s}')

def _ensure_geocoder():
    global _GEOCODER
    if _GEOCODER is None:
        try:
            from geopy.geocoders import Nominatim
            _GEOCODER = Nominatim(user_agent='bersona_geocoder')
        except Exception as e:
            _GEOCODER = False  # 标记不可用

def geocode_china_location(admin_str: str) -> Optional[tuple]:
    """地理编码中国地点（省市格式）。使用 OpenStreetMap Nominatim。

    参数: admin_str 如 '浙江省杭州市'
    返回: (lat, lon) 或 None
    说明: 在线请求，有速率限制；结果缓存以减少重复查询。
    """
    parsed = parse_admin_location(admin_str)
    key = parsed['province'] + parsed['city']
    if key in _GEOCODE_CACHE:
        return _GEOCODE_CACHE[key]
    _ensure_geocoder()
    if not _GEOCODER:
        return None
    query = f"{parsed['city']},{parsed['province']},China"
    try:
        loc = _GEOCODER.geocode(query)
        if loc:
            coord = (loc.latitude, loc.longitude)
            _GEOCODE_CACHE[key] = coord
            return coord
    except Exception:
        return None
    return None



import re
from pathlib import Path

import bersona

def test_version_matches_pyproject():
    pkg_version = bersona.__version__
    pyproject = Path(__file__).resolve().parents[1] / 'pyproject.toml'
    text = pyproject.read_text(encoding='utf-8')
    m = re.search(r'^version\s*=\s*"([0-9]+\.[0-9]+\.[0-9]+)"', text, re.MULTILINE)
    assert m, 'Version not found in pyproject.toml'
    assert m.group(1) == pkg_version, f"Mismatch: pyproject={m.group(1)} package={pkg_version}"